"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTween = void 0;
const vue_1 = require("vue");
const femtotween_1 = require("femtotween");
const extendRefWithStop = (source, stop) => {
    return Object.defineProperty(source, "stop", {
        value: stop,
        enumerable: true,
        writable: true,
    });
};
const setupTween = (from, to, options, callback) => {
    const tweeningValue = (0, vue_1.ref)(from);
    const reactivityCallback = (newValue) => {
        tweeningValue.value =
            null == options.precision
                ? newValue
                : Number.parseFloat(newValue.toFixed(options.precision));
        callback === null || callback === void 0 ? void 0 : callback(newValue);
    };
    const stop = (0, femtotween_1.tween)(from, to, reactivityCallback, options);
    return extendRefWithStop(tweeningValue, stop);
};
const useTweenEx = (from, to, options, callback) => {
    const tweenValue = (0, vue_1.ref)(from.value);
    let innerTweenValue = (0, vue_1.ref)(from.value);
    let innerTweenValueWatchHandler;
    (0, vue_1.watch)([from, to], (value, oldValue, onCleanup) => {
        onCleanup(() => {
            var _a;
            innerTweenValueWatchHandler === null || innerTweenValueWatchHandler === void 0 ? void 0 : innerTweenValueWatchHandler();
            (_a = tweenValue.stop) === null || _a === void 0 ? void 0 : _a.call(tweenValue);
        });
        innerTweenValue = setupTween(from.value, to.value, options, callback);
        tweenValue.stop = innerTweenValue.stop;
        innerTweenValueWatchHandler = (0, vue_1.watch)(innerTweenValue, () => {
            tweenValue.value = innerTweenValue.value;
        }, { immediate: true });
    }, { immediate: true });
    return tweenValue;
};
const _useTween = (source, options, callback) => {
    const from = (0, vue_1.ref)(source.value);
    const to = (0, vue_1.ref)(source.value);
    let tweenedValue = null;
    (0, vue_1.watch)(source, () => {
        if (tweenedValue) {
            from.value = tweenedValue.value;
            to.value = source.value;
        }
    }, { immediate: true });
    tweenedValue = useTweenEx(from, to, options, callback);
    return tweenedValue;
};
/**
 * Composable to use {@link https://github.com/pearofducks/femtoTween femtoTween's} tween function in a (Vue 3 Composition API) reactive way
 *
 * @param source
 * @param options Tweening options.
 * @param callback Callback function that will be called on each tween-frame
 * @returns Ref that updates on each tween-frame; also exposes a 'stop' function to stop tweening
 */
const useTween = (source, options, callback) => {
    if ("number" === typeof source.value) {
        return _useTween(source, options, callback);
    }
    const sourceValues = source.value.map((value) => (0, vue_1.ref)(value));
    const tweenedValues = (0, vue_1.reactive)(sourceValues.map((value) => {
        return _useTween(value, options, callback);
    }));
    const unWrappedTweenedValues = (0, vue_1.computed)(() => {
        return tweenedValues.map((value) => value.value);
    });
    (0, vue_1.watch)(source, (newSources) => {
        newSources.forEach((value, index) => {
            sourceValues[index].value = value;
        });
    }, { deep: true });
    return extendRefWithStop(unWrappedTweenedValues, () => {
        tweenedValues.forEach((value) => {
            value.stop();
        });
    });
};
exports.useTween = useTween;
